
/******************************************************************************
 *
 *  CSignKeyWndTask.c
 *
 *	Sign key window task classes for the SignWriter key editor.
 *
 *  Copyright 1996 The Center for Sutton Movement Writing. All rights reserved.
 *
 ******************************************************************************/
 


#include "CSignKeyWndTask.h"
#include "CSignKeyWndGrid.h"
#include "CSymWindow.h"
#include "KeyEditConsts.h"
#include "Global.h"
#include "Keys.h"
#include "LongCoordinates.h"
#include "Symbols.h"
#include "SymCode.h"
#include "TBUtilities.h"




/*** Global Variables ***/


extern short		gClicks;				/* Single or double click */
						
	
extern void	  		*gClipRec;				/* Clipboard */
extern void	  		*gScratchRec;
extern ObjType		 gScratchType;


extern CSymWindow	*gSymWind;				/* Symbol window access */





/********************************  Clear  **********************************/


/***
 * ISignKeyWndClear
 *
 ***/
 
void CSignKeyWndClear::ISignKeyWndClear (short nameIndex, CSignKeyWndGrid *gridPane, 
 														  Byte keyCode, short keyVar)		
{
	CTask::ITask(nameIndex);
	
	itsGridPane = gridPane;
	itsKeyCode  = keyCode;
	itsKeyVar   = keyVar;
	
}	



/***
 * Do (override)
 *
 * Clear the selected symbol or key variation box.
 * Save original in the global scratch record.
 *
 ***/

void CSignKeyWndClear::Do()

{
	Box	 	  rec;
	ObjType   selType;
	short	  selInx;
	
	
	GetSignKeyVar(itsKeyCode, itsKeyVar, &rec);
	
	GetSelectType(rec, &selType);
	GetSelectSym(rec, &selInx);
	
	
	if (gScratchRec != NULL)
		DisposKey(gScratchType, gScratchRec);
	
	CopySignKeyVar(rec, (Box *) &gScratchRec);
	gScratchType = selType;


	if (selType == box)
		ClearSignKeyVar(rec);				
 	else 
 		DeleteSym(rec, selInx);
 		
		
	SetSelectType(rec, none);
	
	itsGridPane->UpdateKeyVar(itsKeyVar);	

}
		


/***
 * Undo (override)
 *
 *	Exchange the current selection
 *  with the global scratch record.
 *
 ***/

void CSignKeyWndClear::Undo()

{
	Box	 rec;
	
	
	GetSignKeyVar(itsKeyCode, itsKeyVar, &rec);
	SetSignKeyVar(itsKeyCode, itsKeyVar, (Box) gScratchRec);
	gScratchRec = rec;
					
	itsGridPane->UpdateKeyVar(itsKeyVar);
	
}




/********************************  Paste  **********************************/


/***
 * ISignKeyWndPaste
 *
 ***/
 
void CSignKeyWndPaste::ISignKeyWndPaste (CSignKeyWndGrid *gridPane, 
										 Byte keyCode, short keyVar)		
{
	CTask::ITask(iPASTE);
	
	itsGridPane = gridPane;
	itsKeyCode  = keyCode;
	itsKeyVar   = keyVar;
	
}	
	


/***
 * Do (override)
 *
 * Paste the clipboard item into the current selection.
 * Save original in the global scratch record.
 *
 * If the clipboard item is a symbol, add it to the 
 * selected box (if there is room).
 * If the clipboard item is a box, replace the
 * selected box with the clipboard item.
 *
 ***/

void CSignKeyWndPaste::Do()

{
	Box	 	  rec;
	ObjType   selType, clipType;
	short	  selInx, clipInx;
	short	  symCount, junk;
	
	
	 GetSignKeyVar(itsKeyCode, itsKeyVar, &rec);
	
	 GetSelectType(rec, &selType);
	 GetSelectSym(rec, &selInx);
				
	 GetSelectType((Box) gClipRec, &clipType);
	 GetSelectSym((Box) gClipRec, &clipInx); 
	
	
	if (gScratchRec != NULL)
		DisposKey(gScratchType, gScratchRec);
				
				
	if (clipType == sym) {								// clipboard holds symbol...
	
		
		CopySignKeyVar(rec, (Box *) &gScratchRec);
		gScratchType = selType;


		switch (selType) {
		
			case none:
			
				GetSymCount(rec, &symCount);
				
				if (symCount >= KEY_SYMS) {					// key is full 
				
					PositionDialog('ALRT', ALRTkeyFull);	
					junk = Alert(ALRTkeyFull, NULL);	
					
					return;
				}	
				break;
					
			case box:										// clear current selection
			
				ClearSignKeyVar(rec);
				break;
						
			case sym:
				
				DeleteSym(rec, selInx);
				break;				
		}
					
		CopySym((Box) gClipRec, clipInx, rec);				// paste symbol
		
	
 	} else {    										// clipboard holds box...
 	
 	
 		gScratchRec  = rec;									// move current to scratch
 		gScratchType = selType;
	
		CopySignKeyVar((Box) gClipRec, &rec);				// copy clip to current
 		SetSelectType(rec, none);
 			
 		SetSignKeyVar(itsKeyCode, itsKeyVar, rec);			
 		
	}
			
	itsGridPane->UpdateKeyVar(itsKeyVar);			

}
	
	

/***
 * Undo (override)
 *
 *	Exchange the current selection
 *  with the global scratch record.
 *
 ***/

void CSignKeyWndPaste::Undo()

{
	Box	 rec;
	
	
	GetSignKeyVar(itsKeyCode, itsKeyVar, &rec);
	SetSignKeyVar(itsKeyCode, itsKeyVar, (Box) gScratchRec);
	gScratchRec = rec;
	
	itsGridPane->UpdateKeyVar(itsKeyVar);	

}




/*********************************  Add  ***********************************/


/***
 * ISignKeyWndAdd
 *
 ***/
 
void CSignKeyWndAdd::ISignKeyWndAdd (CSignKeyWndGrid *gridPane, 
									 Byte keyCode, short keyVar)	
{
	CTask::ITask(iADD);
	
	itsGridPane = gridPane;
	itsKeyCode  = keyCode;
	itsKeyVar   = keyVar;
	
}	
	


/***
 * Do (override)
 *
 * Add symbol selected in sym window to the 
 * current selection.  Save original in the
 * global scratch record.  Center symbol
 * in the key box -- leave it selected.
 *
 ***/

void CSignKeyWndAdd::Do()

{
	Box	 	  rec;
	ObjType   selType;
	short	  selInx;
	short	  symNum, symCount, junk;
	short	  symXPos, symYPos;
	LongPt	  symFrame, unused;
	Byte	  *symBase;	
	
	
	GetSignKeyVar(itsKeyCode, itsKeyVar, &rec);
	
	GetSelectType(rec, &selType);
	GetSelectSym(rec, &selInx);
	
	if (gScratchRec != NULL)
		DisposKey(gScratchType, gScratchRec);
		
	CopySignKeyVar(rec, (Box *) &gScratchRec);
	gScratchType = selType;


	switch (selType) {
	
		case none:
		
			GetSymCount(rec, &symCount);
			
			if (symCount >= KEY_SYMS) {					// key is full 
			
				PositionDialog('ALRT', ALRTkeyFull);	
				junk = Alert(ALRTkeyFull, NULL);	
				
				return;
			}	
			break;
				
		case box:										// clear current selection
		
			ClearSignKeyVar(rec);
			break;
					
		case sym:
			
			DeleteSym(rec, selInx);
			break;				
	}
			
			
	GetSymCount(rec, &symCount);						// add symbol...					
	GetSymBase(rec, symCount, &symBase);
	
 	SetSelectType(rec, sym);
 	SetSelectSym(rec, symCount);
 	SetSymCount(rec, symCount + 1);

 	
	NewSymCode(symBase);								// build sym code
	
	symNum = gSymWind->SelectedSymbol();
	SetSymNum(symBase, symNum);

	GetCoords(symNum, 0, 0, 0, 0, &symFrame, &unused, FALSE); 
	
	symXPos = (SKCELL_WIDTH  - symFrame.h) / 2;	
	symYPos = (SKCELL_HEIGHT - symFrame.v) / 2;	
	
	SetSymXPos(symBase, symXPos);	
	SetSymYPos(symBase, symYPos);		
	
 		
	itsGridPane->UpdateKeyVar(itsKeyVar);			

}
	


/***
 * Undo (override)
 *
 *	Exchange the current selection
 *  with the global scratch record.
 *
 ***/

void CSignKeyWndAdd::Undo()

{
	Box	 rec;
	
	
	GetSignKeyVar(itsKeyCode, itsKeyVar, &rec);
	SetSignKeyVar(itsKeyCode, itsKeyVar, (Box) gScratchRec);
	gScratchRec = rec;
	
	itsGridPane->UpdateKeyVar(itsKeyVar);	
	
}




/*********************************  Drag  **********************************/



/***
 * ISignKeyWndDrag
 *
 * 	Initialize mouse drag class.  Set class
 *  variables itsGridPane, itsBase (base of symbol code),
 *  and itsFrame (size of symbol being dragged).
 *
 ***/
  
void CSignKeyWndDrag::ISignKeyWndDrag (CSignKeyWndGrid *pane, 
									   short item, Box key, short symInx)
 
{
 	
 	short	code, var, fill, rot, flop, junk;
 	LongPt  notUsed;
 	
 	
	itsItem = item;
	
	itsGridPane = pane;
	GetSymBase(key, symInx, &itsBase);
	
	GetSymInfo(itsBase, &code, &var, &fill, &rot, &flop, &junk, &junk, &junk);
	GetCoords(code, var, fill, rot, flop, &itsFrame, &notUsed, FALSE); 	
	
}



/***
 * BeginTracking (override default method)
 *
 * Prepare for tracking by initially
 * drawing the selected symbol.
 *
 ***/
 
void CSignKeyWndDrag::BeginTracking (LongPt *startPt)

{
	itsGridPane->ToggleSelect(itsItem);					// display sym
}



/***
 * KeepTracking (override default method)
 *
 * Move symbol according to 
 * last mouse movement.
 *
 ***/
 
void CSignKeyWndDrag::KeepTracking (LongPt *currPt, LongPt *prevPt, LongPt *startPt)

{
	short  top, left, right, bottom;
	
	
	if (EqualLongPt(currPt, prevPt))
		return;
	
	
	gClicks = 1;										// drag cancels double click
	
	
	itsGridPane->ToggleSelect(itsItem);					// erase sym
	
	
	GetSymXPos(itsBase, &left);							// get current sym position
	GetSymYPos(itsBase, &top);								 
							
							
	left += (currPt->h - prevPt->h);					// offset by mouse movement			
	top  += (currPt->v - prevPt->v);					
		
	right  = left + itsFrame.h;
	bottom = top  + itsFrame.v;
	
	
	if (left < 0) left = 0;								// check bounds
	if (top  < 0) top  = 0;
	
	if (right  >= SKCELL_WIDTH)  left = SKCELL_WIDTH  - itsFrame.h - 1;
	if (bottom >= SKCELL_HEIGHT) top  = SKCELL_HEIGHT - itsFrame.v - 1;


	SetSymXPos(itsBase, left);							// update sym position
	SetSymYPos(itsBase, top);	
	
	
	itsGridPane->ToggleSelect(itsItem);					// redraw updated sym
							 								
}
